@@ -21,7 +21,6 @@
 Built as a sets of pep8 checks using flake8.
 """
 
-import ConfigParser
 import gettext
 import imp
 import logging
@@ -32,14 +31,22 @@ import sys
 import tokenize
 import traceback
 
-import d2to1.util
+import pbr.util
 import pep8
 
+from hacking import config
+
 # Don't need this for testing
 logging.disable('LOG')
 
 # Import tests need to inject _ properly into the builtins
-gettext.install('hacking', unicode=1)
+kwargs = {}
+if sys.version_info[0] < 3:
+    # In Python2, eusnure that the _() that gets installed into built-ins
+    # always returns unicodes. This matches the default behavior under Python
+    # 3, although the keyword arguement is not present in the Python 3 API.
+    kwargs['unicode'] = True
+gettext.install('hacking', **kwargs)
 
 
 def flake8ext(f):
@@ -60,8 +67,18 @@ def flake8ext(f):
 #H8xx git commit messages
 #H9xx other
 
-IMPORT_EXCEPTIONS = ['sqlalchemy', 'migrate', 'nova.db.sqlalchemy.session',
-                     'nova.db.sqlalchemy.migration.versioning_api']
+
+CONF = config.Config('hacking')
+
+
+DEFAULT_IMPORT_EXCEPTIONS = [
+    'sqlalchemy',
+    'migrate',
+]
+
+IMPORT_EXCEPTIONS = CONF.get_multiple('import_exceptions', default=[])
+IMPORT_EXCEPTIONS += DEFAULT_IMPORT_EXCEPTIONS
+
 # Paste is missing a __init__ in top level directory
 START_DOCSTRING_TRIPLE = ['u"""', 'r"""', '"""', "u'''", "r'''", "'''"]
 END_DOCSTRING_TRIPLE = ['"""', "'''"]
@@ -233,11 +250,11 @@ def hacking_except_format_assert(logical_line):
     OpenStack HACKING guide recommends not using assertRaises(Exception...):
     Do not use overly broad Exception type
 
-    Okay: self.assertRaises(NovaException)
-    Okay: self.assertRaises(ExceptionStrangeNotation)
-    H202: self.assertRaises(Exception)
+    Okay: self.assertRaises(NovaException, foo)
+    Okay: self.assertRaises(ExceptionStrangeNotation, foo)
+    H202: self.assertRaises(Exception, foo)
     """
-    if logical_line.startswith("self.assertRaises(Exception)"):
+    if logical_line.startswith("self.assertRaises(Exception,"):
         yield 1, "H202: assertRaises Exception too broad"
 
 
@@ -714,7 +731,7 @@ def hacking_localization_strings(logical_line, tokens):
     gen = check_i18n()
     next(gen)
     try:
-        map(gen.send, tokens)
+        list(map(gen.send, tokens))
         gen.close()
     except LocalizationError as e:
         yield e.args
@@ -788,10 +805,14 @@ class GitCheck(GlobalCheck):
 
     def _get_commit_title(self):
         # Check if we're inside a git checkout
-        subp = subprocess.Popen(
-            ['git', 'rev-parse', '--show-toplevel'],
-            stdout=subprocess.PIPE)
-        gitdir = subp.communicate()[0].rstrip()
+        try:
+            subp = subprocess.Popen(
+                ['git', 'rev-parse', '--show-toplevel'],
+                stdout=subprocess.PIPE)
+            gitdir = subp.communicate()[0].rstrip()
+        except OSError:
+            # "git" was not found
+            return None
 
         if not os.path.exists(gitdir):
             return None
@@ -804,7 +825,7 @@ class GitCheck(GlobalCheck):
 
         if subp.returncode:
             raise Exception("git log failed with code %s" % subp.returncode)
-        return title
+        return title.decode('utf-8')
 
 
 class OnceGitCheckCommitTitleBug(GitCheck):
@@ -849,7 +870,7 @@ class OnceGitCheckCommitTitleLength(GitCheck):
     def run_once(self):
         title = self._get_commit_title()
 
-        if title and len(title.decode('utf-8')) > 72:
+        if title and len(title) > 72:
             return (
                 1, 0,
                 "H802: git commit title ('%s') should be under 50 chars"
@@ -883,25 +904,19 @@ class ProxyChecks(GlobalCheck):
 
     @classmethod
     def add_options(cls, parser):
-        # Abusing this method because of when it gets called
-        if not os.path.exists('tox.ini'):
-            return
-        tox_ini = ConfigParser.RawConfigParser()
-        tox_ini.read('tox.ini')
-        if not tox_ini.has_section('hacking'):
-            return
-
         # We're looking for local checks, so we need to include the local
         # dir in the search path
         sys.path.append('.')
-        if tox_ini.has_option('hacking', 'local-check'):
-            for check_path in set(
-                    tox_ini.get('hacking', 'local-check').split(",")):
-                if check_path.strip():
-                    checker = d2to1.util.resolve_name(check_path)
-                    pep8.register_check(checker)
-        if tox_ini.has_option('hacking', 'local-check-factory'):
-            factory = d2to1.util.resolve_name(
-                tox_ini.get('hacking', 'local-check-factory'))
+
+        local_check = CONF.get_multiple('local-check', default=[])
+        for check_path in set(local_check):
+            if check_path.strip():
+                checker = pbr.util.resolve_name(check_path)
+                pep8.register_check(checker)
+
+        local_check_fact = CONF.get('local-check-factory')
+        if local_check_fact:
+            factory = pbr.util.resolve_name(local_check_fact)
             factory(pep8.register_check)
+
         sys.path.pop()
